"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const Koa = require("koa");
const Router = require("koa-router");
const _ = require("lodash");
const collection_1 = require("./middleware/collection");
const joi = require("joi");
const validation_middleware_1 = require("./middleware/validation_middleware");
class InnotsApp {
    constructor(config, router = new Router(), customMiddlewares) {
        this.config = config;
        this.router = router;
        this.middlewares = Object.assign({}, collection_1.createDefaultMiddlewareCollection(this.config), customMiddlewares);
    }
    bootstrap() {
        return __awaiter(this, void 0, void 0, function* () {
            this.koaAppInstance = new Koa();
            this._enableBodyParser();
            this._enableLogMiddleware();
            this._enableErrorMiddleware();
            this._enableJwtMiddleware();
            this._enableCorsMiddleware();
            this._enableUserAgentMiddleware();
            this.koaAppInstance.use(this.router.routes());
            this.koaAppInstance.use(this.router.allowedMethods());
            this._enableSuccessMiddleware();
            this.koaAppInstance.on('error', this._processError);
            process.on('uncaughtException', this._processUncaughtException);
            yield this._startApp();
        });
    }
    route(method, url, ...args) {
        if (args[0].length === 1) {
            const joiSchemaGenerator = args[0];
            const validationSchema = joiSchemaGenerator.call(this, joi);
            args[0] = this._createValidationMiddleware(validationSchema);
        }
        this.router[method].apply(this.router, [url].concat(args));
    }
    _enableBodyParser() {
        this.koaAppInstance.use(this.middlewares.bodyParser);
    }
    _enableLogMiddleware() {
        if (this.config.enableLogMiddleware) {
            this.koaAppInstance.use(this.middlewares.log);
        }
    }
    _enableErrorMiddleware() {
        this.koaAppInstance.use(this.middlewares.error);
    }
    _enableJwtMiddleware() {
        const jwtSecret = _.get(this.config, 'jwt.secret');
        if (jwtSecret) {
            this.koaAppInstance.use(this.middlewares.jwt);
        }
    }
    _enableCorsMiddleware() {
        const enabled = _.get(this.config, 'cors.enabled');
        if (typeof enabled === 'undefined' || enabled !== false) {
            this.koaAppInstance.use(this.middlewares.cors);
        }
    }
    _enableUserAgentMiddleware() {
        if (this.config.userAgent) {
            this.koaAppInstance.use(this.middlewares.userAgent);
        }
    }
    _enableSuccessMiddleware() {
        this.koaAppInstance.use(this.middlewares.success);
    }
    _createValidationMiddleware(schema) {
        return validation_middleware_1.createValidationMiddleware(schema);
    }
    _processError(err, ctx) {
        console.error('REQUEST_ERROR', err, ctx);
    }
    _processUncaughtException(err) {
        console.error('PROCESS_EXCEPTION', err.stack);
    }
    _startApp() {
        return __awaiter(this, void 0, void 0, function* () {
            const host = this.config.host;
            const port = this.config.port;
            let promise;
            if (host) {
                promise = new Promise((resolve, reject) => {
                    this.koaAppInstance.listen(port, host, () => {
                        resolve();
                    });
                });
            }
            else {
                promise = new Promise((resolve, reject) => {
                    this.koaAppInstance.listen(port, () => {
                        resolve();
                    });
                });
            }
            return promise;
        });
    }
}
exports.InnotsApp = InnotsApp;
